@@ -1,7 +1,20 @@
-/* $OpenLDAP: pkg/ldap/libraries/libldap/t61.c,v 1.2.2.1 2003/03/03 17:10:05 kurt Exp $ */
-/*
- * Copyright 2002-2003 The OpenLDAP Foundation, All Rights Reserved.
- * COPYING RESTRICTIONS APPLY, see COPYRIGHT file
+/* $OpenLDAP: pkg/ldap/libraries/libldap/t61.c,v 1.4.2.2 2004/01/01 18:16:30 kurt Exp $ */
+/* This work is part of OpenLDAP Software <http://www.openldap.org/>.
+ *
+ * Copyright 2002-2004 The OpenLDAP Foundation.
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted only as authorized by the OpenLDAP
+ * Public License.
+ *
+ * A copy of this license is available in the file LICENSE in the
+ * top-level directory of the distribution or, alternatively, at
+ * <http://www.OpenLDAP.org/license.html>.
+ */
+/* ACKNOWLEDGEMENTS:
+ * This work was initially developed by Howard Chu for inclusion in
+ * OpenLDAP Software.
  */
 
 /*
@@ -332,7 +345,7 @@ int ldap_t61s_to_utf8s( struct berval *src, struct berval *dst )
 		/* Invalid T.61 characters? */
 		if (!t61_tab[*c]) 
 			return LDAP_INVALID_SYNTAX;
-		if (*c & 0xf0 == 0xc0) {
+		if ((*c & 0xf0) == 0xc0) {
 			int j = *c & 0x0f;
 			/* If this is the end of the string, or if the base
 			 * character is just a space, treat this as a regular
@@ -368,7 +381,7 @@ int ldap_t61s_to_utf8s( struct berval *src, struct berval *dst )
 		return LDAP_NO_MEMORY;
 
 	for (i=0,c=(unsigned char *)src->bv_val; i < src->bv_len; c++,i++) {
-		if (*c & 0xf0 == 0xc0) {
+		if ((*c & 0xf0) == 0xc0) {
 			int j = *c & 0x0f;
 			/* If this is the end of the string, or if the base
 			 * character is just a space, treat this as a regular
