@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/add.c,v 1.162.2.11 2004/04/19 18:15:14 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/add.c,v 1.162.2.12 2004/05/21 02:11:38 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2004 The OpenLDAP Foundation.
@@ -625,7 +625,7 @@ slap_entry2mods(
 		for ( i = 0; i < count; i++ ) {
 			ber_dupbv(mod->sml_values+i, a_new->a_vals+i); 
 			if ( mod->sml_nvalues ) {
-				ber_dupbv( mod->sml_nvalues+i, a_new->a_vals+i ); 
+				ber_dupbv( mod->sml_nvalues+i, a_new->a_nvals+i ); 
 			} 
 		}
 
