@@ -1,5 +1,5 @@
 /* search.c - search operation */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/search.c,v 1.136.2.17 2004/04/12 20:29:57 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-bdb/search.c,v 1.136.2.18 2004/06/16 18:19:57 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2000-2004 The OpenLDAP Foundation.
@@ -37,10 +37,10 @@ static int search_candidates(
 	ID	*ids,
 	ID	*scopes );
 
-static void send_pagerequest_response( 
+static void send_paged_response( 
 	Operation *op,
 	SlapReply *rs,
-	ID  lastid,
+	ID  *lastid,
 	int tentries );
 
 /* Dereference aliases for a single alias entry. Return the final
@@ -524,7 +524,9 @@ bdb_do_search( Operation *op, SlapReply *rs, Operation *sop,
 		ei_root.bei_e = &e_root;
 		ei_root.bei_parent = &ei_root;
 		e_root.e_private = &ei_root;
-		e_root.e_id = 1;
+		e_root.e_id = 0;
+		e_root.e_nname.bv_val="";
+		e_root.e_name.bv_val="";
 		ei = &ei_root;
 		rs->sr_err = LDAP_SUCCESS;
 	} else {
@@ -736,9 +738,9 @@ dn2entry_retry:
 	}
 
 	/* if not root and candidates exceed to-be-checked entries, abort */
-	if ( sop->ors_limit	/* isroot == TRUE */
-			&& sop->ors_limit->lms_s_unchecked != -1
-			&& BDB_IDL_N(candidates) > (unsigned) sop->ors_limit->lms_s_unchecked )
+	if ( sop->ors_limit	/* isroot == TRUE */ &&
+		sop->ors_limit->lms_s_unchecked != -1 &&
+		BDB_IDL_N(candidates) > (unsigned) sop->ors_limit->lms_s_unchecked )
 	{
 		rs->sr_err = LDAP_ADMINLIMIT_EXCEEDED;
 		send_ldap_result( sop, rs );
@@ -746,13 +748,14 @@ dn2entry_retry:
 		goto done;
 	}
 
-	if ( sop->ors_limit == NULL	/* isroot == FALSE */
-			|| !sop->ors_limit->lms_s_pr_hide ) {
+	if ( sop->ors_limit == NULL	/* isroot == FALSE */ ||
+		!sop->ors_limit->lms_s_pr_hide )
+	{
 		tentries = BDB_IDL_N(candidates);
 	}
 
-	if ( get_pagedresults(sop) ) {
-		if ( sop->o_pagedresults_state.ps_cookie == 0 ) {
+	if ( get_pagedresults(sop) > SLAP_NO_CONTROL ) {
+		if ( (ID)( sop->o_pagedresults_state.ps_cookie ) == 0 ) {
 			id = bdb_idl_first( candidates, &cursor );
 
 		} else {
@@ -763,20 +766,25 @@ dn2entry_retry:
 				goto done;
 			}
 			for ( id = bdb_idl_first( candidates, &cursor );
-				id != NOID && id <= (ID)( sop->o_pagedresults_state.ps_cookie );
-				id = bdb_idl_next( candidates, &cursor ) ) /* empty */;
+				id != NOID &&
+					id <= (ID)( sop->o_pagedresults_state.ps_cookie );
+				id = bdb_idl_next( candidates, &cursor ) )
+			{
+				/* empty */;
+			}
 		}
+
 		if ( cursor == NOID ) {
 #ifdef NEW_LOGGING
 			LDAP_LOG ( OPERATION, RESULTS, 
 				"bdb_search: no paged results candidates\n", 
-			0, 0, 0 );
+				0, 0, 0 );
 #else
 			Debug( LDAP_DEBUG_TRACE, 
 				"bdb_search: no paged results candidates\n",
 				0, 0, 0 );
 #endif
-			send_pagerequest_response( sop, rs, lastid, 0 );
+			send_paged_response( sop, rs, &lastid, 0 );
 
 			rs->sr_err = LDAP_OTHER;
 			goto done;
@@ -873,7 +881,9 @@ loop_begin:
 		}
 
 		/* check time limit */
-		if ( sop->ors_tlimit != -1 && slap_get_time() > stoptime ) {
+		if ( sop->ors_tlimit != SLAP_NO_LIMIT
+				&& slap_get_time() > stoptime )
+		{
 			rs->sr_err = LDAP_TIMELIMIT_EXCEEDED;
 			rs->sr_ref = rs->sr_v2ref;
 			send_ldap_result( sop, rs );
@@ -980,7 +990,7 @@ id2entry_retry:
 #endif
 		case LDAP_SCOPE_SUBTREE: {
 			EntryInfo *tmp;
-			for (tmp = BEI(e); tmp->bei_parent;
+			for (tmp = BEI(e); tmp;
 				 tmp = tmp->bei_parent ) {
 				if ( tmp->bei_id == base.e_id ) {
 					scopeok = 1;
@@ -1071,6 +1081,7 @@ id2entry_retry:
 			} else {
 				rs->sr_err = LDAP_COMPARE_TRUE;
 			}
+
 		} else {
 			if ( sop->o_sync_mode & SLAP_SYNC_REFRESH ) {
 				rc_sync = test_filter( sop, rs->sr_entry, &cookief );
@@ -1089,6 +1100,7 @@ id2entry_retry:
 #endif
 						}
 						entry_sync_state = LDAP_SYNC_ADD;
+
 					} else {
 						if ( no_sync_state_change ) {
 							goto loop_continue;
@@ -1096,6 +1108,7 @@ id2entry_retry:
 						entry_sync_state = LDAP_SYNC_PRESENT;
 					}
 				}
+
 			} else {
 				rs->sr_err = test_filter( sop,
 					rs->sr_entry, sop->oq_search.rs_filter );
@@ -1105,7 +1118,7 @@ id2entry_retry:
 		if ( rs->sr_err == LDAP_COMPARE_TRUE ) {
 			/* check size limit */
 			if ( --sop->ors_slimit == -1 &&
-					sop->o_sync_slog_size == -1 )
+				sop->o_sync_slog_size == -1 )
 			{
 				if (!IS_PSEARCH) {
 					bdb_cache_return_entry_r( bdb->bi_dbenv,
@@ -1120,10 +1133,9 @@ id2entry_retry:
 				goto done;
 			}
 
-			if ( get_pagedresults(sop) ) {
+			if ( get_pagedresults(sop) > SLAP_NO_CONTROL ) {
 				if ( rs->sr_nentries >= sop->o_pagedresults_size ) {
-					send_pagerequest_response( sop, rs,
-						lastid, tentries );
+					send_paged_response( sop, rs, &lastid, tentries );
 					goto done;
 				}
 				lastid = id;
@@ -1155,6 +1167,7 @@ id2entry_retry:
 							}
 							if (psid_e != NULL) free (psid_e);
 						}
+
 						if ( ps_type == LDAP_PSEARCH_BY_ADD ) {
 							entry_sync_state = LDAP_SYNC_ADD;
 						} else if ( ps_type == LDAP_PSEARCH_BY_DELETE ) {
@@ -1171,6 +1184,7 @@ id2entry_retry:
 							rs->sr_err = LDAP_OTHER;
 							goto done;
 						}
+
 						if ( sop->o_sync_slog_size != -1 ) {
 							if ( entry_sync_state == LDAP_SYNC_DELETE ) {
 								result = slap_add_session_log( op, sop, e );
@@ -1198,6 +1212,7 @@ id2entry_retry:
 							ctrls[num_ctrls] = NULL;
 							rs->sr_ctrls = NULL;
 						}
+
 					} else if ( ps_type == LDAP_PSEARCH_BY_PREMODIFY ) {
 						struct psid_entry* psid_e;
 						psid_e = (struct psid_entry *) ch_calloc(1,
@@ -1217,6 +1232,7 @@ id2entry_retry:
 							ps_type, 0, 0);
 #endif
 					}
+
 				} else {
 					if ( sop->o_sync_mode & SLAP_SYNC_REFRESH ) {
 						if ( rc_sync == LDAP_COMPARE_TRUE ) { /* ADD */
@@ -1233,6 +1249,7 @@ id2entry_retry:
 							sl_free( ctrls[--num_ctrls], sop->o_tmpmemctx );
 							ctrls[num_ctrls] = NULL;
 							rs->sr_ctrls = NULL;
+
 						} else { /* PRESENT */
 							if ( sync_send_present_mode ) {
 								result = slap_build_syncUUID_set( sop,
@@ -1259,10 +1276,12 @@ id2entry_retry:
 										syncUUID_set_cnt = 0;
 									}
 								}
+
 							} else {
 								result = 1;
 							}
 						}
+
 					} else {
 						rs->sr_attrs = sop->oq_search.rs_attrs;
 						rs->sr_ctrls = NULL;
@@ -1278,15 +1297,17 @@ id2entry_retry:
 				case 1:		/* entry not sent */
 					break;
 				case -1:	/* connection closed */
-					if (!IS_PSEARCH)
-					bdb_cache_return_entry_r(bdb->bi_dbenv,
-						&bdb->bi_cache, e, &lock);
+					if (!IS_PSEARCH) {
+						bdb_cache_return_entry_r(bdb->bi_dbenv,
+							&bdb->bi_cache, e, &lock);
+					}
 					e = NULL;
 					rs->sr_entry = NULL;
 					rs->sr_err = LDAP_OTHER;
 					goto done;
 				}
 			}
+
 		} else {
 #ifdef NEW_LOGGING
 			LDAP_LOG ( OPERATION, RESULTS,
@@ -1339,6 +1360,7 @@ nochange:
 					rs->sr_ctrls = NULL;
 					slap_send_syncinfo( sop, rs,
 						LDAP_TAG_SYNC_REFRESH_PRESENT, &cookie, 1, NULL, 0 );
+
 				} else {
 					if ( !no_sync_state_change ) {
 						int slog_found = 0;
@@ -1348,7 +1370,8 @@ nochange:
 						{
 							if ( ps_list->o_sync_slog_size > 0 ) {
 								if ( ps_list->o_sync_state.sid ==
-									sop->o_sync_state.sid ) {
+									sop->o_sync_state.sid )
+								{
 									slog_found = 1;
 									break;
 								}
@@ -1363,6 +1386,7 @@ nochange:
 						}
 						ldap_pvt_thread_rdwr_runlock( &bdb->bi_pslist_rwlock );
 					}
+
 					rs->sr_err = LDAP_SUCCESS;
 					rs->sr_rspoid = LDAP_SYNC_INFO;
 					rs->sr_ctrls = NULL;
@@ -1370,9 +1394,8 @@ nochange:
 						LDAP_TAG_SYNC_REFRESH_DELETE, &cookie, 1, NULL, 0 );
 				}
 
-				if ( cookie.bv_val ) {
-					ch_free( cookie.bv_val );
-				}
+				if ( cookie.bv_val ) ch_free( cookie.bv_val );
+
 			} else {
 				/* refreshOnly mode */
 				struct berval cookie;
@@ -1382,6 +1405,7 @@ nochange:
 				if ( sync_send_present_mode ) {
 					slap_build_sync_done_ctrl( sop, rs, ctrls,
 						num_ctrls++, 1, &cookie, LDAP_SYNC_REFRESH_PRESENTS );
+
 				} else {
 					if ( !no_sync_state_change ) {
 						int slog_found = 0;
@@ -1403,6 +1427,7 @@ nochange:
 						}
 						ldap_pvt_thread_rdwr_runlock( &bdb->bi_pslist_rwlock );
 					}
+
 					slap_build_sync_done_ctrl( sop, rs, ctrls,
 						num_ctrls++, 1, &cookie, LDAP_SYNC_REFRESH_DELETES );
 				}
@@ -1421,12 +1446,17 @@ nochange:
 				ctrls[num_ctrls] = NULL;
 				if ( cookie.bv_val ) ch_free( cookie.bv_val );	
 			}
+
 		} else {
 			rs->sr_ctrls = NULL;
 			rs->sr_ref = rs->sr_v2ref;
 			rs->sr_err = (rs->sr_v2ref == NULL) ? LDAP_SUCCESS : LDAP_REFERRAL;
 			rs->sr_rspoid = NULL;
-			send_ldap_result( sop, rs );
+			if ( get_pagedresults(sop) > SLAP_NO_CONTROL ) {
+				send_paged_response( sop, rs, NULL, 0 );
+			} else {
+				send_ldap_result( sop, rs );
+			}
 		}
 	}
 
@@ -1670,26 +1700,26 @@ static int search_candidates(
 }
 
 static void
-send_pagerequest_response( 
+send_paged_response( 
 	Operation	*op,
 	SlapReply	*rs,
-	ID		lastid,
+	ID		*lastid,
 	int		tentries )
 {
 	LDAPControl	ctrl, *ctrls[2];
 	BerElementBuffer berbuf;
 	BerElement	*ber = (BerElement *)&berbuf;
-	struct berval	cookie = BER_BVNULL;
+	struct berval	cookie = BER_BVC( "" );
 	PagedResultsCookie respcookie;
 
 #ifdef NEW_LOGGING
 	LDAP_LOG ( OPERATION, ENTRY,
-		"send_pagerequest_response: lastid: (0x%08lx) "
+		"send_paged_response: lastid: (0x%08lx) "
 		"nentries: (0x%081x)\n", 
-		lastid, rs->sr_nentries, NULL );
+		lastid ? *lastid : 0, rs->sr_nentries, NULL );
 #else
-	Debug(LDAP_DEBUG_ARGS, "send_pagerequest_response: lastid: (0x%08lx) "
-		"nentries: (0x%081x)\n", lastid, rs->sr_nentries, NULL );
+	Debug(LDAP_DEBUG_ARGS, "send_paged_response: lastid: (0x%08lx) "
+		"nentries: (0x%081x)\n", lastid ? *lastid : 0, rs->sr_nentries, NULL );
 #endif
 
 	ctrl.ldctl_value.bv_val = NULL;
@@ -1698,11 +1728,18 @@ send_pagerequest_response(
 
 	ber_init2( ber, NULL, LBER_USE_DER );
 
-	respcookie = ( PagedResultsCookie )lastid;
+	if ( lastid ) {
+		respcookie = ( PagedResultsCookie )(*lastid);
+		cookie.bv_len = sizeof( respcookie );
+		cookie.bv_val = (char *)&respcookie;
+
+	} else {
+		respcookie = ( PagedResultsCookie )0;
+	}
+
 	op->o_conn->c_pagedresults_state.ps_cookie = respcookie;
-	op->o_conn->c_pagedresults_state.ps_count = op->o_pagedresults_state.ps_count + rs->sr_nentries;
-	cookie.bv_len = sizeof( respcookie );
-	cookie.bv_val = (char *)&respcookie;
+	op->o_conn->c_pagedresults_state.ps_count =
+		op->o_pagedresults_state.ps_count + rs->sr_nentries;
 
 	/*
 	 * FIXME: we should consider sending an estimate of the entries
@@ -1720,6 +1757,7 @@ send_pagerequest_response(
 	rs->sr_ctrls = ctrls;
 	rs->sr_err = LDAP_SUCCESS;
 	send_ldap_result( op, rs );
+	rs->sr_ctrls = NULL;
 
 done:
 	(void) ber_free_buf( ber );
