@@ -1,5 +1,5 @@
 /* init.c - initialize ldap backend */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldap/init.c,v 1.43.2.6 2004/03/17 20:59:58 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldap/init.c,v 1.43.2.10 2004/04/12 18:20:13 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2003-2004 The OpenLDAP Foundation.
@@ -31,7 +31,7 @@
 #include "slap.h"
 #include "back-ldap.h"
 
-#ifdef SLAPD_LDAP_DYNAMIC
+#if SLAPD_LDAP == SLAPD_MOD_DYNAMIC
 
 int init_module(int argc, char *argv[]) {
     BackendInfo bi;
@@ -44,7 +44,7 @@ int init_module(int argc, char *argv[]) {
     return 0;
 }
 
-#endif /* SLAPD_LDAP_DYNAMIC */
+#endif /* SLAPD_LDAP */
 
 int
 ldap_back_initialize(
@@ -148,7 +148,7 @@ ldap_back_db_init(
 
 	li->be = be;
 	be->be_private = li;
-	be->be_flags |= SLAP_BFLAG_NOLASTMOD;
+	SLAP_DBFLAGS(be) |= SLAP_DBFLAG_NOLASTMOD;
 
 	return 0;
 }
@@ -199,6 +199,10 @@ ldap_back_db_destroy(
 			ch_free(li->url);
 			li->url = NULL;
 		}
+		if ( li->lud ) {
+			ldap_free_urldesc( li->lud );
+			li->lud = NULL;
+		}
 		if (li->binddn.bv_val) {
 			ch_free(li->binddn.bv_val);
 			li->binddn.bv_val = NULL;
