@@ -1,5 +1,5 @@
 /* search.c - ldap backend search function */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldap/search.c,v 1.113.2.12 2004/05/21 02:11:39 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-ldap/search.c,v 1.113.2.13 2004/06/16 18:19:57 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1999-2004 The OpenLDAP Foundation.
@@ -79,7 +79,7 @@ ldap_back_search(
 	/* should we check return values? */
 	if (op->ors_deref != -1)
 		ldap_set_option( lc->ld, LDAP_OPT_DEREF, (void *)&op->ors_deref);
-	if (op->ors_tlimit != -1) {
+	if (op->ors_tlimit != SLAP_NO_LIMIT) {
 		tv.tv_sec = op->ors_tlimit;
 		tv.tv_usec = 0;
 	} else {
