@@ -1,5 +1,5 @@
 /* database.c - deals with database subsystem */
-/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/database.c,v 1.30.2.7 2004/03/22 17:33:28 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/back-monitor/database.c,v 1.30.2.8 2004/04/06 18:16:02 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2001-2004 The OpenLDAP Foundation.
@@ -126,7 +126,7 @@ monitor_subsys_database_init(
 			return( -1 );
 		}
 		
-		if ( be->be_flags & SLAP_BFLAG_MONITOR ) {
+		if ( SLAP_MONITOR(be) ) {
 			attr_merge( e, slap_schema.si_ad_monitorContext,
 					be->be_suffix, be->be_nsuffix );
 			attr_merge( e_database, slap_schema.si_ad_monitorContext,
