@@ -1,5 +1,5 @@
 /* backglue.c - backend glue routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/backglue.c,v 1.60.2.15 2004/06/04 03:39:43 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/backglue.c,v 1.60.2.16 2004/06/16 18:19:55 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2001-2004 The OpenLDAP Foundation.
@@ -194,7 +194,9 @@ glue_back_response ( Operation *op, SlapReply *rs )
 
 	switch(rs->sr_type) {
 	case REP_SEARCH:
-		if ( gs->slimit != -1 && rs->sr_nentries >= gs->slimit ) {
+		if ( gs->slimit != SLAP_NO_LIMIT
+				&& rs->sr_nentries >= gs->slimit )
+		{
 			rs->sr_err = gs->err = LDAP_SIZELIMIT_EXCEEDED;
 			return -1;
 		}
@@ -313,14 +315,14 @@ glue_back_search ( Operation *op, SlapReply *rs )
 				continue;
 			if (!dnIsSuffix(&gi->n[i].be->be_nsuffix[0], &b1->be_nsuffix[0]))
 				continue;
-			if (tlimit0 != -1) {
+			if (tlimit0 != SLAP_NO_LIMIT) {
 				op->ors_tlimit = stoptime - slap_get_time ();
 				if (op->ors_tlimit <= 0) {
 					rs->sr_err = gs.err = LDAP_TIMELIMIT_EXCEEDED;
 					break;
 				}
 			}
-			if (slimit0 != -1) {
+			if (slimit0 != SLAP_NO_LIMIT) {
 				op->ors_slimit = slimit0 - rs->sr_nentries;
 				if (op->ors_slimit < 0) {
 					rs->sr_err = gs.err = LDAP_SIZELIMIT_EXCEEDED;
@@ -347,12 +349,20 @@ glue_back_search ( Operation *op, SlapReply *rs )
 				op->o_req_ndn = op->o_bd->be_nsuffix[0];
 				rs->sr_err = op->o_bd->be_search(op, rs);
 
+			} else if (scope0 == LDAP_SCOPE_SUBTREE &&
+				dn_match(&op->o_bd->be_nsuffix[0], &ndn))
+			{
+				rs->sr_err = op->o_bd->be_search( op, rs );
+
 			} else if (scope0 == LDAP_SCOPE_SUBTREE &&
 				dnIsSuffix(&op->o_bd->be_nsuffix[0], &ndn))
 			{
 				op->o_req_dn = op->o_bd->be_suffix[0];
 				op->o_req_ndn = op->o_bd->be_nsuffix[0];
 				rs->sr_err = op->o_bd->be_search( op, rs );
+				if ( rs->sr_err == LDAP_NO_SUCH_OBJECT ) {
+					gs.err = LDAP_SUCCESS;
+				}
 
 			} else if (dnIsSuffix(&ndn, &op->o_bd->be_nsuffix[0])) {
 				rs->sr_err = op->o_bd->be_search( op, rs );
