@@ -1,5 +1,5 @@
 /* config.c - configuration file handling routines */
-/* $OpenLDAP: pkg/ldap/servers/slapd/config.c,v 1.228.2.14 2004/04/15 20:45:28 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/config.c,v 1.228.2.18 2004/06/16 18:19:55 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2004 The OpenLDAP Foundation.
@@ -118,7 +118,6 @@ read_config( const char *fname, int depth )
 	struct berval vals[2];
 	char *replicahost;
 	LDAPURLDesc *ludp;
-	static int lastmod = 1;
 	static BackendInfo *bi = NULL;
 	static BackendDB	*be = NULL;
 
@@ -1338,7 +1337,7 @@ read_config( const char *fname, int depth )
 #ifdef NEW_LOGGING
 					LDAP_LOG( CONFIG, CRIT, "%s: line %d: "
 						"unknown feature %s in \"allow <features>\" line.\n",
-						fname, lineno, cargv[1] );
+						fname, lineno, cargv[i] );
 #else
 					Debug( LDAP_DEBUG_ANY, "%s: line %d: "
 						"unknown feature %s in \"allow <features>\" line\n",
@@ -2183,18 +2182,36 @@ read_config( const char *fname, int depth )
 
 				return( 1 );
 			}
+
+			if ( be == NULL ) {
+#ifdef NEW_LOGGING
+				LDAP_LOG( CONFIG, INFO, "%s: line %d: lastmod"
+					" line must appear inside a database definition\n",
+					fname, lineno , 0 );
+#else
+				Debug( LDAP_DEBUG_ANY, "%s: line %d: lastmod"
+					" line must appear inside a database definition\n",
+					fname, lineno, 0 );
+#endif
+				return 1;
+
+			} else if ( SLAP_NOLASTMODCMD(be) ) {
+#ifdef NEW_LOGGING
+				LDAP_LOG( CONFIG, INFO, "%s: line %d: lastmod"
+					" not available for %s database\n",
+					fname, lineno , be->bd_info->bi_type );
+#else
+				Debug( LDAP_DEBUG_ANY, "%s: line %d: lastmod"
+					" not available for %s databases\n",
+					fname, lineno, be->bd_info->bi_type );
+#endif
+				return 1;
+			}
+
 			if ( strcasecmp( cargv[1], "on" ) == 0 ) {
-				if ( be ) {
-					SLAP_DBFLAGS(be) &= ~SLAP_DBFLAG_NOLASTMOD;
-				} else {
-					lastmod = 1;
-				}
+				SLAP_DBFLAGS(be) &= ~SLAP_DBFLAG_NOLASTMOD;
 			} else {
-				if ( be ) {
-					SLAP_DBFLAGS(be) |= SLAP_DBFLAG_NOLASTMOD;
-				} else {
-					lastmod = 0;
-				}
+				SLAP_DBFLAGS(be) |= SLAP_DBFLAG_NOLASTMOD;
 			}
 
 #ifdef SIGHUP
@@ -3173,51 +3190,48 @@ parse_syncrepl_line(
 		} else if ( !strncasecmp( cargv[ i ],
 			INTERVALSTR, sizeof( INTERVALSTR ) - 1 ) )
 		{
+			char *hstr;
+			char *mstr;
+			char *dstr;
+			char *sstr;
+			int dd, hh, mm, ss;
 			val = cargv[ i ] + sizeof( INTERVALSTR );
-			if ( si->si_type == LDAP_SYNC_REFRESH_AND_PERSIST ) {
-				si->si_interval = 0;
-			} else {
-				char *hstr;
-				char *mstr;
-				char *dstr;
-				char *sstr;
-				int dd, hh, mm, ss;
-				dstr = val;
-				hstr = strchr( dstr, ':' );
-				if ( hstr == NULL ) {
-					fprintf( stderr, "Error: parse_syncrepl_line: "
-						"invalid interval \"%s\"\n", val );
-					return 1;
-				}
-				*hstr++ = '\0';
-				mstr = strchr( hstr, ':' );
-				if ( mstr == NULL ) {
-					fprintf( stderr, "Error: parse_syncrepl_line: "
-						"invalid interval \"%s\"\n", val );
-					return 1;
-				}
-				*mstr++ = '\0';
-				sstr = strchr( mstr, ':' );
-				if ( sstr == NULL ) {
-					fprintf( stderr, "Error: parse_syncrepl_line: "
-						"invalid interval \"%s\"\n", val );
-					return 1;
-				}
-				*sstr++ = '\0';
-
-				dd = atoi( dstr );
-				hh = atoi( hstr );
-				mm = atoi( mstr );
-				ss = atoi( sstr );
-				if (( hh > 24 ) || ( hh < 0 ) ||
-					( mm > 60 ) || ( mm < 0 ) ||
-					( ss > 60 ) || ( ss < 0 ) || ( dd < 0 )) {
-					fprintf( stderr, "Error: parse_syncrepl_line: "
-						"invalid interval \"%s\"\n", val );
-					return 1;
-				}
-				si->si_interval = (( dd * 24 + hh ) * 60 + mm ) * 60 + ss;
+			dstr = val;
+			hstr = strchr( dstr, ':' );
+			if ( hstr == NULL ) {
+				fprintf( stderr, "Error: parse_syncrepl_line: "
+					"invalid interval \"%s\"\n", val );
+				return 1;
 			}
+			*hstr++ = '\0';
+			mstr = strchr( hstr, ':' );
+			if ( mstr == NULL ) {
+				fprintf( stderr, "Error: parse_syncrepl_line: "
+					"invalid interval \"%s\"\n", val );
+				return 1;
+			}
+			*mstr++ = '\0';
+			sstr = strchr( mstr, ':' );
+			if ( sstr == NULL ) {
+				fprintf( stderr, "Error: parse_syncrepl_line: "
+					"invalid interval \"%s\"\n", val );
+				return 1;
+			}
+			*sstr++ = '\0';
+
+			dd = atoi( dstr );
+			hh = atoi( hstr );
+			mm = atoi( mstr );
+			ss = atoi( sstr );
+			if (( hh > 24 ) || ( hh < 0 ) ||
+				( mm > 60 ) || ( mm < 0 ) ||
+				( ss > 60 ) || ( ss < 0 ) || ( dd < 0 )) {
+				fprintf( stderr, "Error: parse_syncrepl_line: "
+					"invalid interval \"%s\"\n", val );
+				return 1;
+			}
+			si->si_interval = (( dd * 24 + hh ) * 60 + mm ) * 60 + ss;
+
 			if ( si->si_interval < 0 ) {
 				fprintf( stderr, "Error: parse_syncrepl_line: "
 					"invalid interval \"%ld\"\n",
