@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/controls.c,v 1.72.2.16 2004/05/21 02:11:38 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/controls.c,v 1.72.2.18 2004/06/16 21:53:56 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2004 The OpenLDAP Foundation.
@@ -598,6 +598,7 @@ int get_ctrls(
 
 			if (( sc->sc_mask & tagmask ) == tagmask ) {
 				/* available extension */
+				int	rc;
 
 				if( !sc->sc_parse ) {
 					rs->sr_err = LDAP_OTHER;
@@ -605,9 +606,12 @@ int get_ctrls(
 					goto return_results;
 				}
 
-				rs->sr_err = sc->sc_parse( op, rs, c );
-				assert( rs->sr_err != LDAP_UNAVAILABLE_CRITICAL_EXTENSION );
-				if( rs->sr_err != LDAP_SUCCESS ) goto return_results;
+				rc = sc->sc_parse( op, rs, c );
+				assert( rc != LDAP_UNAVAILABLE_CRITICAL_EXTENSION );
+				if ( rc ) {
+					rs->sr_err = rc;
+					goto return_results;
+				}
 
 				if ( sc->sc_mask & SLAP_CTRL_FRONTEND ) {
 					/* kludge to disable backend_control() check */
@@ -633,6 +637,7 @@ int get_ctrls(
 			rs->sr_text = "critical extension is not recognized";
 			goto return_results;
 		}
+next_ctrl:;
 	}
 
 return_results:
@@ -909,15 +914,29 @@ static int parsePagedResults (
 	} else {
 		/* Initial request.  Initialize state. */
 		op->o_pagedresults_state.ps_cookie = 0;
-		op->o_pagedresults_state.ps_id = NOID;
 		op->o_pagedresults_state.ps_count = 0;
 	}
 
 	op->o_pagedresults_size = size;
 
-	op->o_pagedresults = ctrl->ldctl_iscritical
-		? SLAP_CRITICAL_CONTROL
-		: SLAP_NONCRITICAL_CONTROL;
+	/* NOTE: according to RFC 2696 3.:
+
+    If the page size is greater than or equal to the sizeLimit value, the
+    server should ignore the control as the request can be satisfied in a
+    single page.
+	 
+	 * NOTE: this assumes that the op->ors_slimit be set
+	 * before the controls are parsed.     
+	 */
+	if ( op->ors_slimit > 0 && size >= op->ors_slimit ) {
+		op->o_pagedresults = SLAP_IGNORED_CONTROL;
+
+	} else if ( ctrl->ldctl_iscritical ) {
+		op->o_pagedresults = SLAP_CRITICAL_CONTROL;
+
+	} else {
+		op->o_pagedresults = SLAP_NONCRITICAL_CONTROL;
+	}
 
 	return LDAP_SUCCESS;
 }
