@@ -1,5 +1,5 @@
 /* limits.c - routines to handle regex-based size and time limits */
-/* $OpenLDAP: pkg/ldap/servers/slapd/limits.c,v 1.30.2.10 2004/04/19 20:13:20 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/limits.c,v 1.30.2.11 2004/06/16 18:19:56 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2004 The OpenLDAP Foundation.
@@ -1001,11 +1001,11 @@ limits_check( Operation *op, SlapReply *rs )
 		op->ors_limit = NULL;
 
 		if ( op->ors_tlimit == 0 ) {
-			op->ors_tlimit = -1;
+			op->ors_tlimit = SLAP_NO_LIMIT;
 		}
 
 		if ( op->ors_slimit == 0 ) {
-			op->ors_slimit = -1;
+			op->ors_slimit = SLAP_NO_LIMIT;
 		}
 
 	/* if not root, get appropriate limits */
@@ -1048,7 +1048,7 @@ limits_check( Operation *op, SlapReply *rs )
 		}
 
 		/* if paged results is requested */	
-		if ( get_pagedresults( op ) ) {
+		if ( get_pagedresults( op ) > SLAP_NO_CONTROL ) {
 			int	slimit = -2;
 			int	pr_total;
 
@@ -1084,7 +1084,7 @@ limits_check( Operation *op, SlapReply *rs )
 			if ( pr_total == -1 ) {
 				slimit = -1;
 
-			} else if ( pr_total > 0 && ( op->ors_slimit == -1 || op->ors_slimit > pr_total ) ) {
+			} else if ( pr_total > 0 && ( op->ors_slimit == SLAP_NO_LIMIT || op->ors_slimit > pr_total ) ) {
 				rs->sr_err = LDAP_ADMINLIMIT_EXCEEDED;
 				send_ldap_result( op, rs );
 				rs->sr_err = LDAP_SUCCESS;
