@@ -1,4 +1,4 @@
-/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/pcache.c,v 1.10.2.9 2004/06/04 03:39:43 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/overlays/pcache.c,v 1.10.2.11 2004/06/16 18:19:58 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 2003-2004 The OpenLDAP Foundation.
@@ -869,8 +869,8 @@ remove_query_data (
 	op->o_req_ndn = op->o_bd->be_nsuffix[0];
 	op->ors_scope = LDAP_SCOPE_SUBTREE;
 	op->ors_deref = LDAP_DEREF_NEVER;
-	op->ors_slimit = -1;
-	op->ors_tlimit = -1;
+	op->ors_slimit = SLAP_NO_LIMIT;
+	op->ors_tlimit = SLAP_NO_LIMIT;
 	op->ors_filter = &filter;
 	op->ors_filterstr.bv_val = filter_str;
 	op->ors_filterstr.bv_len = strlen(filter_str);
@@ -1867,6 +1867,9 @@ proxy_cache_open(
 	int rc = 0;
 
 	if ( cm->db.bd_info->bi_db_open ) {
+		cm->db.be_pending_csn_list = (struct be_pcl *)
+							ch_calloc( 1, sizeof( struct be_pcl ));
+		LDAP_TAILQ_INIT( cm->db.be_pending_csn_list );
 		rc = cm->db.bd_info->bi_db_open( &cm->db );
 	}
 
