@@ -1,5 +1,5 @@
 /* schema_init.c - init builtin schema */
-/* $OpenLDAP: pkg/ldap/servers/slapd/schema_init.c,v 1.297.2.13 2004/04/12 18:13:21 kurt Exp $ */
+/* $OpenLDAP: pkg/ldap/servers/slapd/schema_init.c,v 1.297.2.17 2004/06/16 18:19:56 kurt Exp $ */
 /* This work is part of OpenLDAP Software <http://www.openldap.org/>.
  *
  * Copyright 1998-2004 The OpenLDAP Foundation.
@@ -873,8 +873,11 @@ uniqueMemberNormalize(
 
 	assert( SLAP_MR_IS_VALUE_OF_SYNTAX( usage ));
 
-	ber_dupbv( &out, val );
-	if( out.bv_len != 0 ) {
+	ber_dupbv_x( &out, val, ctx );
+	if( BER_BVISEMPTY( &out ) ) {
+		*normalized = out;
+
+	} else {
 		struct berval uid = BER_BVNULL;
 
 		if( out.bv_val[out.bv_len-1] == 'B'
@@ -884,7 +887,7 @@ uniqueMemberNormalize(
 			uid.bv_val = strrchr( out.bv_val, '#' );
 
 			if( uid.bv_val == NULL ) {
-				free( out.bv_val );
+				sl_free( out.bv_val, ctx );
 				return LDAP_INVALID_SYNTAX;
 			}
 
@@ -898,7 +901,7 @@ uniqueMemberNormalize(
 		rc = dnNormalize( 0, NULL, NULL, &out, normalized, ctx );
 
 		if( rc != LDAP_SUCCESS ) {
-			free( out.bv_val );
+			sl_free( out.bv_val, ctx );
 			return LDAP_INVALID_SYNTAX;
 		}
 
@@ -918,7 +921,7 @@ uniqueMemberNormalize(
 			normalized->bv_val[normalized->bv_len] = '\0';
 		}
 
-		free( out.bv_val );
+		sl_free( out.bv_val, ctx );
 	}
 
 	return LDAP_SUCCESS;
@@ -1749,7 +1752,7 @@ IA5StringNormalize(
 
 		} else if ( casefold ) {
 			/* Most IA5 rules require casefolding */
-			*q++ = TOLOWER(*p++);
+			*q++ = TOLOWER(*p); p++;
 
 		} else {
 			*q++ = *p++;
@@ -1981,11 +1984,14 @@ integerBitOrMatch(
 
 	/* safe to assume integers are NUL terminated? */
 	lValue = SLAP_STRTOL(value->bv_val, NULL, 10);
-	if(( lValue == SLAP_LONG_MIN || lValue == SLAP_LONG_MAX ) && errno == ERANGE ) {
+	if(( lValue == SLAP_LONG_MIN || lValue == SLAP_LONG_MAX ) &&
+		errno == ERANGE )
+	{
 		return LDAP_CONSTRAINT_VIOLATION;
 	}
 
-	lAssertedValue = SLAP_STRTOL(((struct berval *)assertedValue)->bv_val, NULL, 10);
+	lAssertedValue = SLAP_STRTOL( ((struct berval *)assertedValue)->bv_val,
+		NULL, 10);
 	if(( lAssertedValue == SLAP_LONG_MIN || lAssertedValue == SLAP_LONG_MAX )
 		&& errno == ERANGE )
 	{
@@ -2790,9 +2796,14 @@ firstComponentNormalize(
 	void *ctx )
 {
 	int rc;
-	struct berval oid;
+	struct berval comp;
 	ber_len_t len;
 
+	if( SLAP_MR_IS_VALUE_OF_ASSERTION_SYNTAX( usage )) {
+		ber_dupbv_x( normalized, val, ctx );
+		return LDAP_SUCCESS;
+	}
+
 	if( val->bv_len < 3 ) return LDAP_INVALID_SYNTAX;
 
 	if( val->bv_val[0] != '(' /*')'*/ &&
@@ -2810,26 +2821,26 @@ firstComponentNormalize(
 	}
 
 	/* grab next word */
-	oid.bv_val = &val->bv_val[len];
+	comp.bv_val = &val->bv_val[len];
 	len = val->bv_len - len;
-	for( oid.bv_len=0;
-		!ASCII_SPACE(oid.bv_val[oid.bv_len]) && oid.bv_len < len;
-		oid.bv_len++ )
+	for( comp.bv_len=0;
+		!ASCII_SPACE(comp.bv_val[comp.bv_len]) && comp.bv_len < len;
+		comp.bv_len++ )
 	{
 		/* empty */
 	}
 
 	if( mr == slap_schema.si_mr_objectIdentifierFirstComponentMatch ) {
-		rc = numericoidValidate( NULL, &oid );
+		rc = numericoidValidate( NULL, &comp );
 	} else if( mr == slap_schema.si_mr_integerFirstComponentMatch ) {
-		rc = integerValidate( NULL, &oid );
+		rc = integerValidate( NULL, &comp );
 	} else {
 		rc = LDAP_INVALID_SYNTAX;
 	}
 	
 
 	if( rc == LDAP_SUCCESS ) {
-		ber_dupbv_x( normalized, &oid, ctx );
+		ber_dupbv_x( normalized, &comp, ctx );
 	}
 
 	return rc;
